import socket
import time
import paramiko
import ConfigParser
from foxha.utils import Utils


class EnvironmentNode(object):
    def __init__(self, address, user, password, port=22):
        self.address = address
        self.user = user
        self.password = password
        self.port = port


class BuildEnvironment(object):
    def __init__(self, path, keyfile):
        config = ConfigParser.ConfigParser()
        config.read(path)

        self.group = config.get('env', 'group')
        self.description = config.get('env', 'description')
        self.vip_address = config.get('env', 'vip_address')
        self.mysql_adm_user = config.get('env', 'master_mysql_user')
        self.mysql_repl_user = config.get('env', 'slave_mysql_user')

        self.master = config.get('env', 'master')
        self.replication = config.get('env', 'slave')

        cipher = Utils.parse_key_file(keyfile)
        self.master_node = EnvironmentNode(
            self.master,
            config.get('env', 'master_ssh_user'),
            cipher.decrypt(config.get('env', 'master_ssh_password')),
            config.get('env', 'master_ssh_port')
        )
        self.replication_node = EnvironmentNode(
            self.replication,
            config.get('env', 'slave_ssh_user'),
            cipher.decrypt(config.get('env', 'slave_ssh_password')),
            config.get('env', 'slave_ssh_port')
        )

        self.nodes = []
        self.nodes.append(self.master_node)
        self.nodes.append(self.replication_node)


class RemoteHostInterface(object):
    def __init__(self, host, user, password, port=22):
        self._host = host
        self._user = user
        self._password = password
        self.__ssh_client = self.__get_ssh_client()
        self._port = int(port)

    def __get_ssh_client(self,):
        return paramiko.SSHClient()

    def __establish_ssh_connection(self):
        self.__ssh_client.connect(
            self._host, username=self._user, password=self._password, port=self._port
        )

    def __close_ssh_connection(self):
        self.__ssh_client.close()

    def __handle_ssh_keys(self):
        #self.__ssh_client.load_system_host_keys()
        self.__ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    def __exec_remote_command(self, command):
        return self.__ssh_client.exec_command(command)

    def __get_connection_exception_handlers(self):
        return (
            paramiko.ssh_exception.AuthenticationException, socket.error,
            paramiko.ssh_exception.SSHException,
            paramiko.ssh_exception.BadHostKeyException,
            paramiko.ssh_exception.NoValidConnectionsError
        )

    def _exec_remote_command(
        self, command, max_connection_retry=1000, wait_between_retires=10
    ):
        self.__handle_ssh_keys()
        exception_handlers = self.__get_connection_exception_handlers()
        for attempt in xrange(max_connection_retry):
            try:
                self.__establish_ssh_connection()
            except exception_handlers:
                time.sleep(wait_between_retires)
            else:
                break

        return self.__exec_remote_command(command)

    def _handle_command_output(self, stdout_handler, stderr_handler):
        exit_status = stdout_handler.channel.recv_exit_status()
        log_stdout = stdout_handler.read()
        log_stderr = stderr_handler.read()

        return exit_status, log_stdout, log_stderr

    def exec_remote_command(self, command):
        _, stdout, stderr = self._exec_remote_command(command)

        if not stdout and not stderr:
            raise Exception(
                'Unable to establish a connection with: {}'.format(self._host)
            )

        return self._handle_command_output(stdout, stderr)



# -*- coding: utf-8 -*-

class Query:
    SHOW_SLAVE_STATUS = "SHOW SLAVE STATUS;"
    SHOW_GLOBAL_STATUS_SLAVE = "SHOW GLOBAL STATUS LIKE 'Slave_running';"
    SHOW_GLOBAL_STATUS_READ = "SHOW GLOBAL VARIABLES LIKE 'read_only';"
    SHOW_MASTER_STATUS = "SHOW MASTER STATUS;"
    SHOW_GLOBAL_SERVER_ID = "SHOW GLOBAL VARIABLES LIKE 'server_id';"
    SQL_MYSQL_VERSION = "SELECT SUBSTRING(@@version,1,3) as mysql_version;"
    SQL_REPO = "SELECT n.servername,n.node_ip,n.node_port,n.mode,g.mysql_adm_user,\
    g.mysql_adm_pass,g.mysql_repl_user,g.mysql_repl_pass FROM repl_nodes n \
    INNER JOIN repl_groups g ON g.group_name = n.group_name WHERE n.group_name='%s'\
    AND n.status='enabled' ORDER BY n.servername;"
    SQL_REPO_FULL = "SELECT n.servername,n.node_ip,n.node_port,n.mode,n.status,\
    g.mysql_adm_user,g.mysql_adm_pass,g.mysql_repl_user, g.mysql_repl_pass \
    FROM repl_nodes n INNER JOIN repl_groups g ON g.group_name = n.group_name \
    WHERE n.group_name='%s' AND n.status!='disabled' ORDER BY n.servername;"
    SQL_LIST = "SELECT g.group_name FROM repl_groups g ORDER BY g.group_name;"
    SQL_LIST_NODES = "SELECT n.group_name,n.servername,n.node_ip,n.status \
    FROM repl_nodes n WHERE n.group_name='%s' ORDER BY n.servername;"
    SQL_NODE_STATUS = "SELECT n.status, n.mode FROM repl_nodes n \
    WHERE n.group_name='%s' and n.node_ip='%s'"
    SQL_CONFIG_GROUP = "SELECT g.group_name, g.description, g.mysql_adm_user,\
    g.mysql_repl_user, g.vip_address FROM repl_groups g where g.group_name = '%s';"
    SQL_CONFIG_NODES = "SELECT g.group_name,n.servername,n.node_ip,n.node_port,n.mode,n.status \
    FROM repl_nodes n INNER JOIN repl_groups g ON g.group_name = n.group_name \
    where n.group_name = '%s' ORDER BY n.servername;"

    SQL_GROUP_EXIST = \
        'SELECT g.group_name FROM repl_nodes n INNER JOIN repl_groups g ' \
        'ON g.group_name = n.group_name WHERE n.group_name = "%s" limit 1;'

    SQL_GROUP_ADDED = "SELECT g.group_name FROM repl_groups g " \
                      "WHERE g.group_name = '%s' limit 1;"

    SQL_NODE_EXIST = "SELECT node_ip FROM repl_nodes where group_name = '%s' AND node_ip = '%s';"
    SQL_NODE_EXIST_WITHOUT_FAILED = "SELECT node_ip FROM repl_nodes where status = 'enabled'\
    AND group_name = '%s' AND node_ip = '%s';"
    SQL_NODES_WITHOUT_WRITEONE = "SELECT n.group_name,n.servername,n.node_ip,n.node_port,\
    n.status,n.mode,g.mysql_adm_user,g.mysql_adm_pass,g.mysql_repl_user, g.mysql_repl_pass \
    FROM repl_nodes n INNER JOIN repl_groups g ON g.group_name = n.group_name \
    WHERE n.status='enabled' and n.group_name='%s' and n.node_ip != '%s' ORDER BY n.servername;"
    SET_MODE = "SET GLOBAL read_only=%s;"
    UPDATE_STATE = "UPDATE repl_nodes n SET n.status='%s' WHERE n.node_ip='%s' and n.group_name='%s';"
    UPDATE_MODE = "UPDATE repl_nodes n SET n.mode='%s' WHERE n.node_ip='%s' and n.group_name='%s';"
    SQL_CHECK_WRITE = "SELECT n.servername,n.node_ip,n.node_port,n.mode,g.mysql_adm_user,\
    g.mysql_adm_pass,g.mysql_repl_user, g.mysql_repl_pass FROM repl_nodes n \
    INNER JOIN repl_groups g ON g.group_name = n.group_name \
    WHERE n.group_name='%s'AND n.status='enabled' AND n.mode='read_write';"
    SQL_GET_CREDENTIAL = "SELECT n.servername,n.node_ip,n.node_port,n.mode,g.mysql_adm_user,\
    g.mysql_adm_pass,g.mysql_repl_user,g.mysql_repl_pass FROM repl_nodes n \
    INNER JOIN repl_groups g ON g.group_name = n.group_name \
    where n.group_name='%s' AND n.node_ip='%s';"
    SQL_STATUS_HEARTBEAT="SELECT DATE_FORMAT(t.ts, '%Y-%m-%d %H:%i:%S') as last_update FROM heartbeat.heartbeat t;"
    SQL_NODE_DETAILS="SELECT n.servername, n.node_port, g.mysql_adm_user, g.mysql_adm_pass \
    FROM repl_nodes n INNER JOIN repl_groups g \
    ON n.group_name = g.group_name where n.group_name = '%s' and n.node_ip = '%s'"

    SQL_INSERT_GROUP = \
        'INSERT INTO repl_groups (group_name, description, vip_address, ' \
        'mysql_adm_user, mysql_adm_pass, mysql_repl_user, mysql_repl_pass) ' \
        'VALUES ("{}", "{}", "{}", "{}", "{}", "{}", "{}");'

    SQL_DELETE_GROUP = 'DELETE FROM repl_groups WHERE group_name = "{}"'

    SQL_INSERT_NODE = \
        'INSERT INTO repl_nodes (group_name, servername, node_ip, ' \
        'node_port, mode, status, timestamp) ' \
        'VALUES ("{}", "{}", "{}", "{}", "{}", "{}", now());'

    SQL_DELETE_NODE = \
        'DELETE FROM repl_nodes ' \
        'WHERE group_name = "{}" AND node_ip = "{}" '

    SQL_STOP_SLAVE = "STOP SLAVE"

    SQL_START_SLAVE = "START SLAVE"



from foxha import fox
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


def test_can_failover_node(environment, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)

    fox.failover(environment.group)

    node = test_utils.get_node_from_list(environment.master, capsys)
    assert node['status'] == 'failed'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_write'


def test_cannot_failover_with_no_write_node(environment, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    fox.failover(environment.group)

    node = test_utils.get_node_from_list(environment.master, capsys)
    assert node['status'] == 'enabled'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_only'


def test_cannot_failover_no_enabled_node(environment, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)
    node = test_utils.disable_node(environment.replication, capsys)
    assert node['status'] == 'disabled'

    assert not fox.failover(environment.group)

    node = test_utils.enable_node(environment.replication, capsys)
    assert node['status'] == 'enabled'



# -*- coding: utf-8 -*-

import ConfigParser
import logging
from logging import handlers
from os import path
from cryptography.fernet import Fernet
from cryptography.fernet import InvalidToken
from .print_format import print_error


class Utils(object):

    @staticmethod
    def get_key_path():
        return path.dirname(__file__) + '/../config/.key'

    @staticmethod
    def get_config_path():
        return path.dirname(__file__) + '/../config/foxha_config.ini'

    @staticmethod
    def parse_config_file(
            cipher_suite, config_file="./config/foxha_config.ini"
    ):
        try:
            repo_host, repo_port, repo_database, repo_user,\
                encrypted_repo_pass =\
                Utils.get_config_values_from_config_file(config_file)
        except (ConfigParser.NoSectionError) as err:
            print_error("Config file error: {}".format(err))
            exit(99)
        except (ConfigParser.NoOptionError) as err:
            print_error("Config file error: {}".format(err))
            exit(99)

        try:
            decrypted_repo_pass = cipher_suite.decrypt(encrypted_repo_pass)
            return repo_host, repo_port, repo_database,\
                repo_user, decrypted_repo_pass
        except InvalidToken as e:
            print_error("ERROR: InvalidToken")
            exit(99)
        except Exception as e:
            print_error("ERROR: %s" % e)
            exit(3)

    @staticmethod
    def get_config_values_from_config_file(config_file):
        config_parser = ConfigParser.ConfigParser()
        config_parser.read(config_file)

        return config_parser.get('repository', 'Host'),\
            config_parser.getint('repository', 'Port'),\
            config_parser.get('repository', 'Database'),\
            config_parser.get('repository', 'User'),\
            config_parser.get('repository', 'Pass')

    @staticmethod
    def parse_key_file(keyfile="./config/.key"):
        try:
            key = Utils.get_key_value_from_key_file(keyfile)
        except IOError:
            error_message = "ERROR: \"{}\" file does not exist or you don`t"\
                " have permission to read it.".format(keyfile)
            print_error(error_message)
            exit(3)

        try:
            return Fernet(key)
        except ValueError as e:
            print_error("ERROR: {} may be empty because {}".format(keyfile, e))
            exit(3)
        except TypeError as e:
            print_error("ERROR: {} is not a valid key because {}".format(key, e))
            exit(3)

    @staticmethod
    def get_key_value_from_key_file(key_file):
        key_file_handler = open(key_file, "r+")
        return key_file_handler.readline().rstrip('\n')

    @staticmethod
    def generate_new_keyfile(keyfile=None):
        if not keyfile:
            keyfile = '.new_keyfile'

        try:
            key = Fernet.generate_key()
            f = open(keyfile, 'w')
            f.write(key + '\n')
            f.close()
            keyfile_path = path.abspath(keyfile)
            print "New keyfile generated at %s" % keyfile_path
        except Exception as e:
            print_error("ERROR: %s" % e)
            exit(3)

    @staticmethod
    def crypt_pass(cipher_suite, password):
        try:
            cipher_text = cipher_suite.encrypt(password)
            print cipher_text
        except InvalidToken as e:
            print_error("ERROR: InvalidToken")
            exit(99)

    @staticmethod
    def decrypt_pass(cipher_suite, password):
        try:
            cipher_text = cipher_suite.decrypt(password)
            print cipher_text
        except InvalidToken as e:
            print_error("ERROR: InvalidToken")
            exit(99)

    @staticmethod
    def logfile(logname, logretention=4):
        #logsize = 10485760 #size = 10*1024*1024 - Default equal to 10MB
        logrotation = 1  # 1 Day
        LOGGER = logging.getLogger()
        try:
            handler = logging.handlers.TimedRotatingFileHandler(
                filename=logname, when='midnight', interval=logrotation, backupCount=logretention)
            formatter = logging.Formatter(
                fmt='%(asctime)-4s - %(name)-4s => %(levelname)-5s %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S')
            handler.setFormatter(formatter)
            LOGGER.addHandler(handler)
            LOGGER.setLevel(logging.INFO)
            return LOGGER
        except IOError:
            print_error("ERROR: Path does not exist or you don`t have "\
            "permission to the logfile \"%s\"" % logname)
            exit(3)



import os
import pytest

from foxha import connection
from foxha.utils import Utils
from test_helpers import BuildEnvironment


@pytest.fixture(scope='module')
def test_dir():
    return os.path.dirname(__file__) + '/'


def test_config_dir():
    return test_dir() + 'config/'


@pytest.fixture(scope='module')
def test_key_path():
    return test_config_dir() + '.test_key'


@pytest.fixture(scope='module')
def test_connection_config_path():
    return test_config_dir() + 'foxha_config.ini'


def test_config_environment():
    return test_config_dir() + 'environment.ini'


def output_log_path():
    return '/tmp/foxha_tests.log'


@pytest.fixture(scope='module')
def environment(path=test_config_environment(), key_path=test_key_path()):
    return BuildEnvironment(path, key_path)


@pytest.fixture(scope='module')
def default_connection():
    return connection.from_config_file(
        Utils.parse_key_file(test_key_path()),
        test_connection_config_path()
    )


@pytest.fixture(scope='module')
def default_logger():
    return Utils.logfile(output_log_path())



import pytest
from foxha import inner_logic
from foxha.query import Query
from foxha.errors import GroupNotFoundError, GroupAlreadyAddedError, \
    NodeAlreadyAddedError, NodeNotFoundError, GroupWithNodesError
import conftest


def clear_database():
    group = GroupEnv()
    node = NodeEnv()
    connection = conftest.default_connection()
    connection.query(Query.SQL_DELETE_NODE.format(node.group, node.ip))
    connection.query(Query.SQL_DELETE_GROUP.format(group.name))


def teardown_module(module):
    clear_database()


def setup_function(module):
    clear_database()


class GroupEnv(object):
    def __init__(self):
        self.name = 'test_env'
        self.description = 'Fake env'
        self.vip_address = 'vip.off.com'
        self.mysql_user = 'one'
        self.mysql_pass = 'two'
        self.relp_user = 'three'
        self.relp_pass = 'four'


@pytest.fixture
def group():
    return GroupEnv()


class NodeEnv(object):
    def __init__(self):
        self.group = 'test_env'
        self.name = 'fake'
        self.ip = '127.0.0.1'
        self.port = 3306
        self.mode = 'read_write'
        self.status = 'enabled'


@pytest.fixture
def node():
    return NodeEnv()


def test_can_add_new_group(default_connection, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )

    assert inner_logic.get_added_group(
        group.name, default_connection
    ).name == group.name


def test_cannot_add_added_group(default_connection, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )

    with pytest.raises(GroupAlreadyAddedError):
        inner_logic.add_group(
            default_connection, group.name, group.description,
            group.vip_address, group.mysql_user, group.mysql_pass,
            group.relp_user, group.relp_pass
        )


def test_can_delete_added_group(default_connection, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )
    assert inner_logic.delete_group(default_connection, group.name)

    with pytest.raises(GroupNotFoundError):
        inner_logic.get_group(group.name, default_connection)


def test_cannot_delete_external_group(default_connection, group):
    with pytest.raises(GroupNotFoundError):
        inner_logic.delete_group(default_connection, group.name)


def test_cannot_delete_group_with_nodes(default_connection, group, node):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )
    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )

    with pytest.raises(GroupWithNodesError):
        inner_logic.delete_group(default_connection, group.name)


def test_can_add_node(default_connection, node, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )

    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )

    added_node = inner_logic.get_node(node.group, node.ip, default_connection)
    assert added_node.group == node.group
    assert added_node.ip == node.ip
    assert added_node.name == node.name


def test_cannot_add_added_node(default_connection, node, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )

    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )

    with pytest.raises(NodeAlreadyAddedError):
        inner_logic.add_node(
            default_connection, node.group, node.name,
            node.ip, node.port, node.mode, node.status
        )


def test_cannot_add_node_external_group(default_connection, node):
    with pytest.raises(GroupNotFoundError):
        inner_logic.add_node(
            default_connection, node.group, node.name,
            node.ip, node.port, node.mode, node.status
        )


def test_can_delete_added_node(default_connection, node, group):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )

    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )
    inner_logic.delete_node(default_connection, node.group, node.ip)

    with pytest.raises(NodeNotFoundError):
        inner_logic.get_node(node.group, node.ip, default_connection)


def test_cannot_delete_external_node(default_connection, node):
    with pytest.raises(NodeNotFoundError):
        inner_logic.delete_node(default_connection, node.group, node.ip)


def test_not_master_master_with_one_node(default_connection, group, node):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )
    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )
    assert not inner_logic.is_master_master(group.name, default_connection)


def test_not_master_master_with_more_than_two_nodes(
        default_connection, group, node
):
    inner_logic.add_group(
        default_connection, group.name, group.description, group.vip_address,
        group.mysql_user, group.mysql_pass, group.relp_user, group.relp_pass
    )
    inner_logic.add_node(
        default_connection, node.group, node.name,
        node.ip, node.port, node.mode, node.status
    )
    inner_logic.add_node(
        default_connection, node.group, 'Fake-2',
        '127.0.0.2', node.port, node.mode, node.status
    )
    inner_logic.add_node(
        default_connection, node.group, 'Fake-3',
        '127.0.0.3', node.port, node.mode, node.status
    )

    is_master_master = inner_logic.is_master_master(
        group.name, default_connection
    )
    assert inner_logic.delete_node(default_connection, node.group, '127.0.0.2')
    assert inner_logic.delete_node(default_connection, node.group, '127.0.0.3')
    assert not is_master_master



from time import sleep
from foxha import fox
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


def test_can_switch_between_nodes(environment, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)

    fox.switchover(environment.group)
    sleep(15)

    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['mode'] == 'read_only'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_write'


def test_cannot_switch_with_no_write_node(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    fox.switchover(environment.group)

    node = test_utils.get_node_from_list(environment.master, capsys)
    assert node['status'] == 'enabled'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_only'


def test_switchover_twice(environment, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)

    fox.switchover(environment.group)
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['mode'] == 'read_only'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_write'

    fox.switchover(environment.group)

    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['mode'] == 'read_write'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_only'



from prettytable import PrettyTable
from print_format import fore_red, fore_green, print_error, print_warning

import inner_logic
from errors import ManyWriteNodesError, NoWriteNodeError, NodeIsDownError,\
    ReplicationNotRunningError, NodeWithDelayError, GroupNotFoundError,\
    IsNotMasterMasterEnvironmentError, NodeNotFoundError

from node import Node


def list_group(connection):
    for group in inner_logic.get_groups(connection):
        print "* {}".format(group.name)
    exit(0)


def list_nodes(group, connection):
    list_table = PrettyTable(["Group_name", "Node", "Status"])
    list_table.header_style = "upper"

    for node in inner_logic.get_nodes(group, connection):
        status = fore_red(node.fox_status)
        if node.is_enabled():
            status = fore_green(node.fox_status)
        list_table.add_row([group, node.address, status])

    print list_table
    exit(0)


def status_nodes(group, logger, connection):
    status_table = PrettyTable(
        ["Group_name",
         "Node",
         "Node_Status",
         "Mode",
         "Repl_Status",
         "Sec_Behind",
         "Master_Host"]
    )
    status_table.header_style = "upper"
    status_table.align = "c"  # Center align

    logger.info('Health of group_name: "{}"'.format(group))

    messages = []
    for node in inner_logic.get_nodes(group, connection):
        if node.is_failed():
            messages.append({
                'text': "Critical -> Node: {} has failed status at "
                        "repository.".format(node.address),
                'Err_Type': 'Critical'})
            logger.critical(
                "{} has failed status at repository.".format(node.address)
            )

        if node.is_mysql_status_up():

            if node.seconds_behind > 0:
                messages.append({
                    'text':
                        "Warning -> Node: {} - Heartbeat is delayed {} "
                        "seconds.".format(
                            node.get_name_ip_port(), str(node.seconds_behind)
                        ),
                    'Err_Type': "Warning"})
                logger.warning(
                    "Node: {} - Heartbeat is delayed {} seconds.".format(
                        node.get_name_ip_port(), node.seconds_behind
                    )
                )

            if node.is_replication_running():
                status_table.add_row([node.group,
                                      node.address,
                                      node.mysql_status,
                                      node.mode,
                                      node.replication_status,
                                      node.seconds_behind,
                                      node.master])
                logger.info(
                    "Node: {}, Mode: {}, Node_status: {}, Replication running,"
                    " Seconds_Behind: {}, Master_Host: {}".format(
                        node.address, node.mode, node.mysql_status,
                        node.seconds_behind, node.master
                    )
                )
            else:
                status_table.add_row([node.group,
                                      node.address,
                                      node.mysql_status,
                                      node.mode,
                                      fore_red(node.replication_status),
                                      " - ",
                                      " - "])
                logger.critical(
                    "Node: {}, Mode: {}, Node_status: {}, "
                    "Replication stopped".format(
                        node.address, node.mode, node.mysql_status
                    )
                )

            if node.mode != node.fox_mode:
                messages.append({
                    'text':
                        'Critical -> Node: {} - Database mode differs '
                        'from repository! Should be {} but is {}.'.format(
                            node.get_name_ip_port(), node.fox_mode, node.mode
                        ),
                    'Err_Type': 'Critical'
                })
                logger.critical(
                    'Database mode on node {} differs from repository! '
                    'Should be {} but is {}.'.format(
                        node.get_name_ip_port(), node.fox_mode, node.mode
                    )
                )
        else:
            status_table.add_row([node.group,
                                  node.address,
                                  fore_red(node.mysql_status, True),
                                  node.fox_mode, '-', '-', '-'])
            messages.append({
                'text': 'Critical -> Node: {} - {}'.format(
                    node.get_name_ip_port(), fore_red(node.mysql_status, True)
                ),
                'Err_Type': "Critical"
            })
            logger.error(
                "Node: {}, Mode: {}, Node_status: {}".format(
                    node.get_name_ip_port(), node.fox_mode, node.mysql_status
                )
            )

    print status_table
    messages.sort(reverse=True)
    for message in messages:
        if message['Err_Type'] == "Critical":
            print_error("{text}".format(**message))
        else:
            print_warning("{text}".format(**message))


def check_write(group, connection, logger):
    try:
        return inner_logic.get_write_node(group, connection)
    except ManyWriteNodesError:
        print_error("Something is wrong! You could not have more than one "
                    "node with read_write mode. Check your configuration!")
        logger.critical(
            "Something is wrong! You could not have more than one node with "
            "read_write mode. Check your configuration!")
        exit(99)
    except NoWriteNodeError:
        logger.info("There isn't any node with the 'read_write' "
                    "mode at group_name: '{}'.".format(group))
    return False


def is_master_master(group, connection, logger):
    if inner_logic.is_master_master(group, connection):
        return True
    else:
        logger.info(
            'Cannot guarantee that replication is '
            'master<->master for group_name: '.format(group)
        )
    return False


def set_read_write(group, node_ip, connection, logger):
    node = Node(group, node_ip, connection)

    try:
        if inner_logic.set_read_write(node, connection):
            logger.info(
                'Node: "{}" defined as "read_write" '
                'at group_name: "{}".'.format(node.ip, node.group)
            )
            return True
    except NodeIsDownError:
        print_error(
            '--set read_write failed. Connection could not '
            'established with the node_ip {}'.format(node_ip)
        )
        logger.error(
            '--set read_write failed. Connection could not '
            'established with the node_ip {}'.format(node_ip)
        )
    except ReplicationNotRunningError:
        print_warning(
            'The replication is stopped at node "{}" and '
            'cannot be set as read_write.'.format(node_ip)
        )
        logger.error(
            'The replication is stopped at node "{}" and '
            'cannot be set as read_write.'.format(node_ip)
        )
    except NodeWithDelayError:
        print_warning(
            'The node "{}" is {} seconds delayed and cannot be set as '
            'read_write.'.format(node_ip, node.seconds_behind)
        )
        logger.error(
            'The node "{}" is {} seconds delayed and cannot be set as '
            'read_write.'.format(node_ip, node.seconds_behind)
        )
    return False


def set_read_only(group, node_ip, connection, logger):
    node = Node(group, node_ip, connection)

    try:
        if inner_logic.set_read_only(node, connection):
            logger.info(
                'Node: "{}" defined as "read_only" '
                'at group_name: "{}"'.format(node.ip, node.group)
            )
            return True
    except NodeIsDownError:
        print_error(
            '--set read_only failed. Connection could not be '
            'established with the node_ip {}'.format(node_ip)
        )
        logger.error(
            '--set read_only failed. Connection could not be '
            'established with the node_ip {}'.format(node_ip)
        )
    return False


def switchover(group, connection, logger):
    try:
        inner_logic.switchover(group, connection)
    except NoWriteNodeError:
        print_warning(
            'There isn\'t any node with the "read_write" mode. '
            'Check your configuration!'
        )
    except IsNotMasterMasterEnvironmentError:
        print_warning(
            'Switch not done! Slave is stopped or it`s not '
            'a master<->master replication.'
        )
        logger.warning(
            'Switch not done! Slave is stopped or it`s not '
            'a master<->master replication.'
        )
    except NodeWithDelayError:
        print_warning(
            'Switch not done! Node is delayed and cannot be set as read_write'
        )
        logger.error(
            'Switch not done! Node is delayed and cannot be set as read_write'
        )


def failover(group, connection, logger):
    try:
        inner_logic.failover(group, connection)
    except NoWriteNodeError:
        print_warning(
            'There isn\'t any node with the "read_write" mode. '
            'Check your configuration!'
        )
    except IsNotMasterMasterEnvironmentError:
        print_warning(
            'You have more than two nodes and we\'re not prepared to that.')
        logger.warning(
            'You have more than two nodes or it`s not a master<->master and '
            'we\'re not prepared to that'
        )
    except NodeIsDownError:
        print_error('ERROR -> The only candidate you have is unavailable!')
        logger.error('ERROR -> The only candidate you have is unavailable!')


def check_group_exist(name, connection, logger):
    try:
        inner_logic.get_group(name, connection)
        return True
    except GroupNotFoundError:
        print_warning(
            'There is no group_name identified by "{}"!'.format(name)
        )
        logger.error(
            'There is no group_name identified by "{}"'.format(name)
        )
        return False


def check_node_exist(group, node_ip, can_be_failed, connection, logger):
    try:
        node = inner_logic.get_node(group, node_ip, connection)
        if not can_be_failed and node.is_failed():
            raise NodeNotFoundError(node_ip, group)

        return True
    except NodeNotFoundError:
        print_warning(
            'The Ip: "{}" does not belongs to the group_name "{}" or '
            'it`s not enabled.'.format(node_ip, group)
        )
        logger.warning(
            'The Ip: "{}" does not belongs to the group_name "{}" or '
            'it`s not enabled.'.format(node_ip, group)
        )
        return False


def start(group_name, connection, logger):
    try:
        inner_logic.start(group_name, connection)
    except NoWriteNodeError:
        print_warning(
            'There isn\'t any node with the "read_write" mode. '
            'Check your configuration!'
        )
        logger.error(
            'FoxHa version cannot be initiated for '
            'group_name: "{}"'.format(group_name)
        )
    except ManyWriteNodesError:
        print_warning(
            'There is many nodes with the "read_write" mode. '
            'Check your configuration!'
        )
        logger.error(
            'FoxHa version cannot be initiated for '
            'group_name: "{}"'.format(group_name)
        )
    except NodeIsDownError:
        print_error('Connection could not established with write node')
        logger.error('Connection could not established with write node')
    else:
        logger.info('FoxHa initiated for group_name: {}'.format(group_name))


def config(group_name, connection, logger):
    try:
        group = inner_logic.get_group(group_name, connection)
    except GroupNotFoundError:
        print_warning(
            'There is no group_name identified by "{}"!'.format(group_name)
        )
        logger.error(
            'There is no group_name identified by "{}"'.format(group_name)
        )
    else:
        print \
            "GROUP_NAME: {}" \
            "\n\tDescription: {} " \
            "\n\tfqdn_vip: {} " \
            "\n\tmysql_adm_user: {}" \
            "\n\tmysql_repl_user: {}".format(
                group.name, group.description, group.vip_address,
                group.mysql_adm_user, group.mysql_repl_user
            )

        for node in group.nodes:
            print \
                "NODE: [{}]" \
                "\n\tip: {}" \
                "\n\tport: {}" \
                "\n\tmode: {}" \
                "\n\tstatus: {}".format(
                    node.name, node.ip, node.port,
                    node.fox_mode, node.fox_status
                )



from contextlib import contextmanager
import _mysql as MySQLdb
import MySQLdb.cursors
from utils import Utils


def from_config_file(cipher=None, config_path=None):
    if not cipher:
        cipher = Utils.parse_key_file(Utils.get_key_path())

    if not config_path:
        config_path = Utils.get_config_path()

    connection_params = Utils.parse_config_file(cipher, config_path)
    return Connection(*connection_params, cipher=cipher)


class Connection(object):
    def __init__(self, host, port, database, user, password, cipher):
        self.host = host
        self.port = port
        self.database = database
        self.user = user
        self.password = password
        self.cipher = cipher

    @contextmanager
    def connect(self):
        client = None
        try:
            client = MySQLdb.connect(
                host=self.host, port=self.port, user=self.user,
                passwd=self.password, db=self.database, connect_timeout=5,
                cursorclass=MySQLdb.cursors.DictCursor, compress=1
            )
            yield client
        finally:
            if client:
                client.close()

    def execute(self, sql_statement):
        with self.connect() as client:
            client.query(sql_statement)
            result = client.store_result()
            if result:
                return result.fetch_row(maxrows=0, how=1)

    def query(self, sql_statement):
        with self.connect() as client:
            client.query(sql_statement)
            client.commit()



import re
import pytest
import time
from foxha import fox
from foxha import formatter
import conftest
from test_helpers import RemoteHostInterface


MYSQL_STOP_CMD = '/etc/init.d/mysql stop'
MYSQL_START_CMD = '/etc/init.d/mysql start'
MYSQL_RESTART_CMD = '/etc/init.d/mysql restart'


def __get_name(node):
    return re.search('.*(\()', node.strip()).group(0)[:-1]


def __get_ip(node):
    ip = re.search('(\().*', node.strip()).group(0)[1:-1]
    if re.search('.*(\:)', ip):
        ip = re.search('.*(\:)', ip).group(0)[:-1]
    return ip


def is_node_in_line(node_ip, line):
    values = line.split('|')
    if len(values) > 2:
        return node_ip in values[2]
    return False


def remove_ansi_format(string):
    return re.sub(r'\x1b[^m]*m', '', str(string.strip()))


def get_node_from_list(node_ip, capsys, group=None):
    if not group:
        group = conftest.environment().group

    with pytest.raises(SystemExit) as exec_info:
        formatter.list_nodes(group, conftest.default_connection())
    assert str(exec_info.value) == '0'

    out, err = capsys.readouterr()
    for line in out.split('\n'):
        if is_node_in_line(node_ip, line):
            data = line
            break

    group, address, status = data.split('|')[1:4]
    ip = __get_ip(address)
    name = __get_name(address)
    status = remove_ansi_format(status)

    return {'group': group.strip(), 'name': name.strip(),
            'ip': ip.strip(), 'status': status}


def __change_node_status(node_ip, group, status, capsys):
    node = get_node_from_list(node_ip, capsys, group)
    if node['status'] != status:
        fox.set(status, node['group'], node['ip'])
        node = get_node_from_list(node_ip, capsys, group)
    return node


def disable_node(node_ip, capsys, group=None):
    return __change_node_status(node_ip, group, 'disabled', capsys)


def enable_node(node_ip, capsys, group=None):
    return __change_node_status(node_ip, group, 'enabled', capsys)


def failed_node(node_ip, capsys, group=None):
    return __change_node_status(node_ip, group, 'failed', capsys)


def get_node_from_status(node_ip, capsys, group=None):
    if not group:
        group = conftest.environment().group
    formatter.status_nodes(
        group, conftest.default_logger(), conftest.default_connection()
    )

    out, err = capsys.readouterr()
    for line in out.split('\n'):

        if is_node_in_line(node_ip, line):
            data = line
            break

    group, address, status, mode, repl_status = data.split('|')[1:6]
    ip = __get_ip(address)
    name = __get_name(address)
    status = remove_ansi_format(status)

    return {
        'group': group.strip(),
        'name': name.strip(),
        'ip': ip.strip(),
        'status': status,
        'mode': mode.strip(),
        'repl_status': repl_status.strip()
    }


def __change_node_mode(node_ip, group, mode, capsys):
    node = get_node_from_status(node_ip, capsys, group)
    if node['mode'] != mode:
        fox.set(mode, node['group'], node['ip'])
        node = get_node_from_status(node_ip, capsys, group)
    return node


def set_read_only(node_ip, capsys, group=None):
    return __change_node_mode(node_ip, group, 'read_only', capsys)


def set_read_write(node_ip, capsys, group=None):
    return __change_node_mode(node_ip, group, 'read_write', capsys)


def reset_database():
    environment = conftest.environment()

    fox.set('enabled', environment.group, environment.master)
    fox.set('enabled', environment.group, environment.replication)
    fox.set('read_only', environment.group, environment.replication)
    fox.set('read_write', environment.group, environment.master)


def connect_database():
    fox.CONNECTION = conftest.default_connection()
    fox.LOGGER = conftest.default_logger()


def activate_nodes():
    environment = conftest.environment()
    __execute_command_in_nodes(environment, MYSQL_RESTART_CMD)


def deactivate_nodes():
    environment = conftest.environment()
    __execute_command_in_nodes(environment, MYSQL_STOP_CMD)


def __execute_command_in_nodes(environment, command):
    for node in environment.nodes:
        __execute_command_in_node(node, command)


def deactivate_node(node):
    return __execute_command_in_node(node, MYSQL_STOP_CMD)


def activate_node(node):
    return __execute_command_in_node(node, MYSQL_START_CMD)


def __execute_command_in_node(node, command):
    remote = RemoteHostInterface(node.address, node.user, node.password, node.port)
    return remote.exec_remote_command(command)


def is_replication_ok(environment, capsys):
    node = get_node_from_status(environment.master, capsys)
    if node['repl_status'] != 'running':
        return False

    node = get_node_from_status(environment.replication, capsys)
    if node['repl_status'] != 'running':
        return False

    return True


def wait_for_replication_ok(environment, capsys):
    for _ in xrange(12):
        if is_replication_ok(environment, capsys):
            break
        time.sleep(5)
    time.sleep(1)



from query import Query
from group import Group
from node import Node
import node_utils
from errors import ManyWriteNodesError, NoWriteNodeError, NodeIsDownError,\
    IsNotMasterMasterEnvironmentError, ReplicationNotRunningError, \
    NodeWithDelayError, GroupNotFoundError, NodeNotFoundError, \
    GroupAlreadyAddedError, NodeAlreadyAddedError, GroupWithNodesError


def get_nodes(group, connection):
    return node_utils.get_all_nodes(group, connection)


def get_groups(connection):
    result = connection.execute(Query.SQL_LIST)
    return [Group(group['group_name'], connection) for group in result]


def get_write_node(group, connection):
    result = connection.execute(Query.SQL_CHECK_WRITE % group)

    if len(result) == 1:
        return Node(group, result[0]['node_ip'], connection)
    elif len(result) > 1:
        raise ManyWriteNodesError(group, len(result))
    else:
        raise NoWriteNodeError(group)


def is_master_master(group, connection):
    result = connection.execute(Query.SQL_REPO % group)
    if len(result) != 2:
        return False

    nodes = get_nodes(group, connection)
    master = nodes[0]
    pair = nodes[1]
    if master.is_replication_running() and pair.is_replication_running():
        same_server = master.server_id == pair.master_server_id
        same_replication = master.master_server_id == pair.server_id
        return same_server and same_replication

    return False


def set_read_write(node, connection):
    if node.is_mysql_status_down():
        raise NodeIsDownError(node.ip)

    try:
        node_write = get_write_node(node.group, connection)
    except NoWriteNodeError:
        node.node_connection.execute(Query.SET_MODE % 'OFF')
        connection.query(
            Query.UPDATE_MODE % ('read_write', node.ip, node.group)
        )
        return True

    if node == node_write:
        node.node_connection.query(Query.SET_MODE % 'OFF')
        return True

    if node.is_replication_stopped():
        raise ReplicationNotRunningError(node.ip)

    if node.seconds_behind > 0:
        raise NodeWithDelayError(node.ip, node.seconds_behind)

    set_read_only(node_write, connection)
    node.node_connection.execute(Query.SET_MODE % 'OFF')
    connection.query(Query.UPDATE_MODE % ('read_write', node.ip, node.group))
    return True


def set_read_only(node, connection):
    if node.is_mysql_status_down():
        raise NodeIsDownError(node.ip)

    node.node_connection.query(Query.SET_MODE % 'ON')
    connection.query(Query.UPDATE_MODE % ('read_only', node.ip, node.group))
    return True


def switchover(group, connection):
    get_write_node(group, connection)

    if not is_master_master(group, connection):
        raise IsNotMasterMasterEnvironmentError(group)

    for node in get_nodes(group, connection):
        if not node.is_read_write():
            if set_read_write(node, connection):
                break

    return True


def failover(group, connection):
    node_write = get_write_node(group, connection)

    nodes = get_nodes(group, connection)

    if len(nodes) != 2:
        raise IsNotMasterMasterEnvironmentError(group)

    connection.query(Query.UPDATE_STATE % ('failed', node_write.ip, group))
    connection.query(Query.UPDATE_MODE % ('read_only', node_write.ip, group))

    for node in nodes:
        if node.is_enabled():
            if node.is_mysql_status_down():
                raise NodeIsDownError(node.ip)

            connection.query(
                Query.UPDATE_MODE % ('read_write', node.ip, group)
            )
            node.node_connection.query(Query.SET_MODE % 'OFF')
            return True
    return False


def get_group(name, connection):
    result = connection.execute(Query.SQL_GROUP_EXIST % name)

    if len(result) == 0:
        raise GroupNotFoundError(name)

    return Group(result[0]['group_name'], connection)


def get_added_group(name, connection):
    result = connection.execute(Query.SQL_GROUP_ADDED % name)
    if len(result) > 0:
        return Group(result[0]['group_name'], connection)


def get_node(group, node_ip, connection):
    result = connection.execute(Query.SQL_NODE_EXIST % (group, node_ip))

    if len(result) == 0:
        raise NodeNotFoundError(node_ip, group)

    return Node(group, node_ip, connection)


def add_group(
        connection, group_name, description, vip_address,
        mysql_user, mysql_password, repl_user, repl_password):

    if get_added_group(group_name, connection):
        raise GroupAlreadyAddedError(group_name)

    connection.query(
        Query.SQL_INSERT_GROUP.format(
            group_name, description, vip_address,
            mysql_user, connection.cipher.encrypt(mysql_password),
            repl_user, connection.cipher.encrypt(repl_password)
        )
    )
    return True


def delete_group(connection, group_name):
    group = get_added_group(group_name, connection)
    if not group:
        raise GroupNotFoundError(group_name)

    if len(group.nodes) > 0:
        raise GroupWithNodesError(group_name, len(group.nodes))

    connection.query(Query.SQL_DELETE_GROUP.format(group_name))
    return True


def add_node(connection, group_name, name, ip, port, mode, status):
    if not get_added_group(group_name, connection):
        raise GroupNotFoundError(group_name)

    try:
        get_node(group_name, ip, connection)
        raise NodeAlreadyAddedError(ip, group_name)
    except NodeNotFoundError:
        pass

    connection.query(
        Query.SQL_INSERT_NODE.format(group_name, name, ip, port, mode, status)
    )
    return True


def delete_node(connection, group_name, node_ip):
    get_node(group_name, node_ip, connection)
    connection.query(Query.SQL_DELETE_NODE.format(group_name, node_ip))
    return True


def start(group_name, connection):
    node_write = get_write_node(group_name, connection)

    if node_write.is_mysql_status_down():
        raise NodeIsDownError(node_write.ip)

    node_write.node_connection.query(Query.SET_MODE % 'OFF')
    return True



import pytest
from foxha import fox


def test_version_message(capsys):
    with pytest.raises(SystemExit) as e:
        fox.main(['--version'])
    assert str(e.value) == '0'

    out, err = capsys.readouterr()
    assert '0.9.0' in err


def test_can_get_config(environment, capsys):
    fox.main(['-c', '-g', environment.group])
    out, err = capsys.readouterr()
    assert 'GROUP_NAME: {}'.format(environment.group) in out


def test_can_get_status(environment, capsys):
    fox.main(['--status', '-g', environment.group])

    out, err = capsys.readouterr()
    assert environment.master in out
    assert environment.replication in out


def test_can_list_nodes(environment, capsys):
    with pytest.raises(SystemExit) as e:
        fox.main(['-l', '-g', environment.group])
    assert str(e.value) == '0'

    out, err = capsys.readouterr()
    assert environment.master in out
    assert environment.replication in out


def test_can_list_groups(environment, capsys):
    with pytest.raises(SystemExit) as e:
        fox.main(['-l'])
    assert str(e.value) == '0'

    out, err = capsys.readouterr()
    assert environment.group in out



import ConfigParser
import pytest


def test_config_file_successfully(
    cipher_suite, utils, test_connection_config_path, config_file_dict
):
    config_file = utils.parse_config_file(
        cipher_suite, test_connection_config_path
    )

    assert config_file[0] == config_file_dict['host']
    assert config_file[1] == config_file_dict['port']
    assert config_file[2] == config_file_dict['database']
    assert config_file[3] == config_file_dict['user']
    assert config_file[4] == cipher_suite.decrypt(config_file_dict['password'])


def test_config_file_throws_no_section_error(
    cipher_suite, utils, capsys, connection_config_without_section
):
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_config_file(
            cipher_suite, connection_config_without_section
        )

    error_message = u"\x1b[38;5;160mConfig file error: No section:"\
        " 'repository'\x1b[0m\n"

    out, err = capsys.readouterr()
    assert error_message == out
    assert str(exec_info.value) == '99'


def test_config_file_throws_no_option_error(
    cipher_suite, utils, capsys, connection_config_without_option
):
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_config_file(
            cipher_suite, connection_config_without_option
        )

    error_message = u"\x1b[38;5;160mConfig file error: No option"\
        " 'database' in section: 'repository'\x1b[0m\n"

    out, err = capsys.readouterr()
    assert error_message == out
    assert str(exec_info.value) == '99'


def test_config_file_throws_invalid_token(
    cipher_suite, utils, capsys, connection_config_with_invalid_token
):
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_config_file(
            cipher_suite, connection_config_with_invalid_token
        )

    error_message = u"\x1b[38;5;160mERROR: InvalidToken\x1b[0m\n"

    out, err = capsys.readouterr()
    assert error_message == out
    assert str(exec_info.value) == '99'


def test_config_file_throws_exception(
    cipher_suite, utils, capsys, test_connection_config_path
):
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_config_file(
            'empty string', test_connection_config_path
        )

    error_message = u"\x1b[38;5;160mERROR: 'str' object has no attribute"\
        " 'decrypt'\x1b[0m\n"

    out, err = capsys.readouterr()
    assert error_message == out
    assert str(exec_info.value) == '3'


def test_get_config_values_from_config_file_successfully(
    cipher_suite, utils, test_connection_config_path, config_file_dict
):
    config_values = utils.get_config_values_from_config_file(
        test_connection_config_path
    )

    assert config_values[0] == config_file_dict['host']
    assert config_values[1] == config_file_dict['port']
    assert config_values[2] == config_file_dict['database']
    assert config_values[3] == config_file_dict['user']
    assert config_values[4] == config_file_dict['password']


def test_get_config_values_from_config_file_throws_no_section_error(
    utils, connection_config_without_section
):
    with pytest.raises(ConfigParser.NoSectionError) as exec_info:
        utils.get_config_values_from_config_file(
            connection_config_without_section
        )

    assert str(exec_info.value) == "No section: 'repository'"


def test_get_config_values_from_config_file_throws_no_option_error(
    utils, connection_config_without_option
):
    with pytest.raises(ConfigParser.NoOptionError) as exec_info:
        utils.get_config_values_from_config_file(
            connection_config_without_option
        )

    assert str(exec_info.value) == "No option 'database' in section: 'repository'"



import pytest
from foxha import inner_logic
from foxha.group import Group
from foxha.query import Query
from foxha.node import LogicalNode
from foxha.errors import GroupNotFoundError, NodeNotFoundError, \
    NoWriteNodeError, ManyWriteNodesError, NodeIsDownError, \
    ReplicationNotRunningError, IsNotMasterMasterEnvironmentError
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


def test_can_list_all_groups(environment, default_connection):
    groups = inner_logic.get_groups(default_connection)
    assert Group(environment.group, default_connection) in groups


def test_cannot_found_external_group_in_list(default_connection):
    groups = inner_logic.get_groups(default_connection)
    assert Group('Strange_Group_Env', default_connection) not in groups


def test_can_find_nodes_in_list(environment, default_connection):
    node_list = inner_logic.get_nodes(environment.group, default_connection)
    env_nodes = environment.nodes
    assert len(env_nodes) == len(node_list)

    for node in env_nodes:
        assert LogicalNode(
            environment.group, node.address, '', default_connection
        ) in node_list


def test_cannot_found_node_for_external_group(environment, default_connection):
    node_list = inner_logic.get_nodes('Strange_Group_Env', default_connection)
    assert len(node_list) == 0


def test_can_get_group(environment, default_connection):
    group = inner_logic.get_group(environment.group, default_connection)
    assert Group(environment.group, default_connection) == group


def test_cannot_get_group_external(environment, default_connection):
    with pytest.raises(GroupNotFoundError):
        inner_logic.get_group('Strange_Group_Env', default_connection)


def test_can_get_node(environment, default_connection):
    node = inner_logic.get_node(
        environment.group, environment.master, default_connection
    )
    assert node.ip == environment.master
    assert node.group == environment.group


def test_cannot_get_node(environment, default_connection):
    with pytest.raises(NodeNotFoundError):
        inner_logic.get_node(
            'Strange_Group_Env', '111.222.333.444', default_connection
        )


def test_cannot_get_node_wrong_group(environment, default_connection):
    with pytest.raises(NodeNotFoundError):
        inner_logic.get_node(
            'Strange_Group_Env', environment.master, default_connection
        )


def test_cannot_get_node_wrong_ip(environment, default_connection):
    with pytest.raises(NodeNotFoundError):
        inner_logic.get_node(
            environment.group, '111.222.333.444', default_connection
        )


def test_can_start_group(environment, default_connection):
    node_write = inner_logic.get_node(
        environment.group, environment.master, default_connection
    )
    node_write.node_connection.query(Query.SET_MODE % 'ON')
    assert node_write.mode != node_write.fox_mode

    assert inner_logic.start(environment.group, default_connection)
    assert node_write.mode == node_write.fox_mode


def test_cannot_start_group_without_write(environment, default_connection):
    node_write = inner_logic.get_node(
        environment.group, environment.master, default_connection
    )
    node_write.fox_connection.query(
        Query.UPDATE_MODE % ('read_only', node_write.ip, node_write.group)
    )
    assert node_write.mode != node_write.fox_mode

    with pytest.raises(NoWriteNodeError):
        inner_logic.start(environment.group, default_connection)


def test_cannot_start_group_with_many_write(environment, default_connection):
    node_repl = inner_logic.get_node(
        environment.group, environment.replication, default_connection
    )
    node_repl.fox_connection.query(
        Query.UPDATE_MODE % ('read_write', node_repl.ip, node_repl.group)
    )
    assert node_repl.mode != node_repl.fox_mode

    with pytest.raises(ManyWriteNodesError):
        inner_logic.start(environment.group, default_connection)

    node_repl.fox_connection.query(
        Query.UPDATE_MODE % ('read_only', node_repl.ip, node_repl.group)
    )
    assert node_repl.mode == node_repl.fox_mode


def test_cannot_start_with_master_node_down(environment, default_connection):
    node_write = inner_logic.get_node(
        environment.group, environment.master, default_connection
    )
    assert node_write.is_mysql_status_up()

    test_utils.deactivate_node(environment.master_node)
    assert node_write.is_mysql_status_down()
    with pytest.raises(NodeIsDownError):
        inner_logic.start(environment.group, default_connection)

    test_utils.activate_node(environment.master_node)


def test_cannot_set_write_without_replication(
        environment, default_connection, capsys
):
    test_utils.wait_for_replication_ok(environment, capsys)
    node_replication = inner_logic.get_node(
        environment.group, environment.replication, default_connection
    )
    assert node_replication.is_replication_running()

    node_replication.node_connection.query(Query.SQL_STOP_SLAVE)
    assert node_replication.is_replication_stopped()

    try:
        inner_logic.set_read_write(node_replication, default_connection)
    except ReplicationNotRunningError:
        assert True
    else:
        assert False, 'DO NOT RAISE ReplicationNotRunningError'
    finally:
        node_replication.node_connection.query(Query.SQL_START_SLAVE)
        test_utils.wait_for_replication_ok(environment, capsys)
        assert node_replication.is_replication_running()


def test_cannot_switch_with_no_master_master(environment, default_connection):
    inner_logic.add_node(
        default_connection, environment.group, 'FakeNode',
        '127.0.0.1', 3306, 'read_only', 'enabled'
    )

    try:
        inner_logic.switchover(environment.group, default_connection)
        assert False
    except IsNotMasterMasterEnvironmentError:
        assert True
    except:
        assert False
    finally:
        inner_logic.delete_node(
            default_connection, environment.group, '127.0.0.1'
        )


def test_cannot_failover_with_no_master_master(environment, default_connection):
    inner_logic.add_node(
        default_connection, environment.group, 'FakeNode',
        '127.0.0.1', 3306, 'read_only', 'enabled'
    )

    try:
        inner_logic.failover(environment.group, default_connection)
        assert False
    except IsNotMasterMasterEnvironmentError:
        assert True
    except:
        assert False
    finally:
        inner_logic.delete_node(
            default_connection, environment.group, '127.0.0.1'
        )


def test_cannot_failover_node_down(environment, default_connection, capsys):
    test_utils.deactivate_node(environment.replication_node)

    try:
        inner_logic.failover(environment.group, default_connection)
        assert False
    except NodeIsDownError:
        assert True
    except:
        assert False
    finally:
        test_utils.activate_node(environment.replication_node)
        test_utils.wait_for_replication_ok(environment, capsys)



import time
import re
import pytest
from foxha import fox
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


def test_can_change_node_to_read_only(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'


def test_cannot_change_to_read_when_already_is_read(environment, capsys):
    node = test_utils.set_read_only(environment.replication, capsys)
    assert node['mode'] == 'read_only'


def test_can_change_node_to_read_write(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    node = test_utils.set_read_write(environment.replication, capsys)
    assert node['mode'] == 'read_write'


@pytest.mark.flaky(reruns=5)
def test_cannot_change_to_write_with_heartbeat_delay(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    time.sleep(2)

    node = test_utils.set_read_write(environment.master, capsys)
    assert node['mode'] == 'read_write'

    fox.set('read_write', environment.group, environment.replication)
    delay_message = 'The node "%s" is (\d+.\d+) seconds delayed and ' \
                    'cannot be set as read_write.' % environment.replication
    out, err = capsys.readouterr()
    assert re.findall(delay_message, out)

    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_only'


def test_cannot_change_to_write_when_already_is_write(environment, capsys):
    node = test_utils.set_read_write(environment.master, capsys)
    assert node['mode'] == 'read_write'


def test_can_enable_node(environment, capsys):
    node = test_utils.disable_node(environment.replication, capsys)
    assert node['status'] == 'disabled'

    node = test_utils.enable_node(environment.replication, capsys)
    assert node['status'] == 'enabled'


def test_can_disable_node_without_write(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    node = test_utils.disable_node(environment.master, capsys)
    assert node['status'] == 'disabled'

    node = test_utils.disable_node(environment.replication, capsys)
    assert node['status'] == 'disabled'


def test_cannot_disable_write_node(environment, capsys):
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['mode'] == 'read_write'

    node = test_utils.disable_node(environment.master, capsys)
    assert node['status'] == 'enabled'


def test_can_disable_read_node(environment, capsys):
    node = test_utils.disable_node(environment.replication, capsys)
    assert node['status'] == 'disabled'


def test_can_fail_without_write(environment, capsys):
    node = test_utils.set_read_only(environment.master, capsys)
    assert node['mode'] == 'read_only'

    node = test_utils.failed_node(environment.master, capsys)
    assert node['status'] == 'failed'

    node = test_utils.failed_node(environment.replication, capsys)
    assert node['status'] == 'failed'


def test_can_fail_read_node(environment, capsys):
    node = test_utils.failed_node(environment.replication, capsys)
    assert node['status'] == 'failed'


def test_failover_when_write_node_failed(environment, capsys):
    node = test_utils.failed_node(environment.master, capsys)
    assert node['status'] == 'failed'

    node = test_utils.get_node_from_list(environment.replication, capsys)
    assert node['status'] == 'enabled'
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_write'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from colored import fore, style


def fore_red(text, bold=False):
    if bold:
        text = (style.BOLD + fore.RED_3B + text + style.RESET)
    else:
        text = (fore.RED_3B + text + style.RESET)
    
    return text


def fore_green(text):
    return fore.SPRING_GREEN_3A + text + style.RESET


def print_warning(text):
    print fore.LIGHT_YELLOW + text + style.RESET


def print_error(text):
    print fore.RED_3B + text + style.RESET


"""
class ForeColor:
    OKGREEN = '\033[92m'
    OKCYAN = '\033[36m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    RESET = '\033[0m'
"""



from foxha import fox
from foxha import formatter
from foxha import inner_logic
from foxha.query import Query
import test_utils


def build_message(group, node):
    return 'The Ip: "{}" does not belongs to the group_name "{}" or ' \
           'it`s not enabled.'.format(node, group)


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


def test_check_group_exist(environment, default_connection, default_logger):
    assert formatter.check_group_exist(
        environment.group, default_connection, default_logger
    )


def test_check_group_not_exist(
        environment, capsys, default_connection, default_logger
):
    assert not formatter.check_group_exist(
        'Strange_Group_Env', default_connection, default_logger
    )
    message = 'There is no group_name identified by "Strange_Group_Env"!'

    out, err = capsys.readouterr()
    assert message in out


def test_check_node_exist(environment):
    assert fox.check_node_exist(
        environment.group, environment.replication, False
    )


def test_check_failed_node_exist(environment, capsys):
    test_utils.failed_node(environment.replication, capsys)
    assert fox.check_node_exist(
        environment.group, environment.replication, True
    )


def test_check_failed_node_not_exist(environment, capsys):
    test_utils.failed_node(environment.replication, capsys)
    assert not fox.check_node_exist(
        environment.group, environment.replication, False
    )
    out, err = capsys.readouterr()
    assert build_message(environment.group, environment.replication) in out


def test_check_node_not_exist_without_failed(environment, capsys):
    assert not fox.check_node_exist(
        'Strange_Group_Env', '987.654.321.666', False
    )
    out, err = capsys.readouterr()
    assert build_message('Strange_Group_Env', '987.654.321.666') in out


def test_check_node_not_exist_with_failed(environment, capsys):
    assert not fox.check_node_exist(
        'Strange_Group_Env', '987.654.321.666', True
    )

    out, err = capsys.readouterr()
    assert build_message('Strange_Group_Env', '987.654.321.666') in out


def test_check_node_not_exist_wrong_group_without_failed(environment, capsys):
    assert not fox.check_node_exist(
        'Strange_Group_Env', environment.replication, False
    )
    out, err = capsys.readouterr()
    assert build_message('Strange_Group_Env', environment.replication) in out


def test_check_node_not_exist_wrong_group_with_failed(environment, capsys):
    assert not fox.check_node_exist(
        'Strange_Group_Env', environment.replication, True
    )

    out, err = capsys.readouterr()
    assert build_message('Strange_Group_Env', environment.replication) in out


def test_check_node_not_exist_wrong_ip_without_failed(environment, capsys):
    assert not fox.check_node_exist(
        environment.group, '123.123.123.123', False
    )
    out, err = capsys.readouterr()
    assert build_message(environment.group, '123.123.123.123') in out


def test_check_node_not_exist_wrong_ip_with_failed(environment, capsys):
    assert not fox.check_node_exist(
        environment.group, '123.123.123.123', True
    )

    out, err = capsys.readouterr()
    assert build_message(environment.group, '123.123.123.123') in out


def test_can_start_group(environment, default_connection, default_logger):
    node_write = formatter.check_write(
        environment.group, default_connection, default_logger
    )
    node_write.node_connection.query(Query.SET_MODE % 'ON')
    assert node_write.mode != node_write.fox_mode

    formatter.start(environment.group, default_connection, default_logger)
    assert node_write.mode == node_write.fox_mode


def test_cannot_start_group_without_write(
        environment, capsys, default_connection, default_logger
):
    node_write = formatter.check_write(
        environment.group, default_connection, default_logger
    )
    node_write.fox_connection.query(
        Query.UPDATE_MODE % ('read_only', node_write.ip, node_write.group)
    )
    assert node_write.mode != node_write.fox_mode

    formatter.start(environment.group, default_connection, default_logger)
    out, err = capsys.readouterr()
    assert \
        'There isn\'t any node with the "read_write" mode. ' \
        'Check your configuration!' in out


def test_cannot_start_group_with_many_write(
        environment, capsys, default_connection, default_logger
):
    node_repl = inner_logic.get_node(
        environment.group, environment.replication, default_connection
    )
    node_repl.fox_connection.query(
        Query.UPDATE_MODE % ('read_write', node_repl.ip, node_repl.group)
    )
    assert node_repl.mode != node_repl.fox_mode

    formatter.start(environment.group, default_connection, default_logger)
    out, err = capsys.readouterr()
    assert 'There is many nodes with the "read_write" mode. ' \
           'Check your configuration!' in out

    node_repl.fox_connection.query(
        Query.UPDATE_MODE % ('read_only', node_repl.ip, node_repl.group)
    )
    assert node_repl.mode == node_repl.fox_mode


def test_cannot_start_with_master_node_down(
        environment, capsys, default_connection, default_logger
):
    node_write = formatter.check_write(
        environment.group, default_connection, default_logger
    )
    assert node_write.is_mysql_status_up()

    test_utils.deactivate_node(environment.master_node)
    assert node_write.is_mysql_status_down()

    formatter.start(environment.group, default_connection, default_logger)
    out, err = capsys.readouterr()
    assert 'Connection could not established with write node' in out

    test_utils.activate_node(environment.master_node)


def test_can_print_config(
        environment, capsys, default_connection, default_logger
):
    formatter.config(environment.group, default_connection, default_logger)
    out, err = capsys.readouterr()

    assert "GROUP_NAME: {}".format(environment.group) in out
    assert "Description: {}".format(environment.description) in out
    assert "fqdn_vip: {}".format(environment.vip_address) in out
    assert "mysql_adm_user: {}".format(environment.mysql_adm_user) in out
    assert "mysql_repl_user: {}".format(environment.mysql_repl_user) in out
    assert out.count('NODE: [') == len(environment.nodes)


def test_cannot_print_config_to_external_node(
        environment, capsys, default_connection, default_logger
):
    formatter.config('Strange_Group_Env', default_connection, default_logger)
    out, err = capsys.readouterr()

    message = 'There is no group_name identified by "Strange_Group_Env"!'
    assert message in out



import pytest
from foxha.node import LogicalNode
import foxha.fox
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


@pytest.fixture
def logical_replication_node(environment, default_connection):
    return LogicalNode(
        environment.group, environment.replication, 'master',
        default_connection
    )


def test_can_create_node(environment, logical_replication_node):
    assert logical_replication_node.group == environment.group
    assert logical_replication_node.ip == environment.replication
    assert logical_replication_node.name == 'master'
    assert logical_replication_node.address == 'master(%s)' % environment.replication
    assert logical_replication_node.fox_status == 'enabled'
    assert logical_replication_node.fox_mode == 'read_only'


def test_status_enabled(environment, logical_replication_node):
    assert logical_replication_node.fox_status == 'enabled'
    assert logical_replication_node.is_enabled()
    assert not logical_replication_node.is_disabled()
    assert not logical_replication_node.is_failed()


def test_status_disabled(environment, logical_replication_node):
    foxha.fox.set('disabled', environment.group, environment.replication)
    assert logical_replication_node.fox_status == 'disabled'
    assert not logical_replication_node.is_enabled()
    assert logical_replication_node.is_disabled()
    assert not logical_replication_node.is_failed()


def test_status_failed(environment, logical_replication_node):
    foxha.fox.set('failed', environment.group, environment.replication)
    assert logical_replication_node.fox_status == 'failed'
    assert not logical_replication_node.is_enabled()
    assert not logical_replication_node.is_disabled()
    assert logical_replication_node.is_failed()


def test_mode_read_write(environment, logical_replication_node, capsys):
    test_utils.wait_for_replication_ok(environment, capsys)
    foxha.fox.set_read_write(environment.group, environment.replication)
    assert logical_replication_node.fox_mode == 'read_write'
    assert not logical_replication_node.is_fox_mode_read_only()
    assert logical_replication_node.is_fox_mode_read_write()


def test_mode_read_only(environment, logical_replication_node):
    assert logical_replication_node.fox_mode == 'read_only'
    assert not logical_replication_node.is_fox_mode_read_write()
    assert logical_replication_node.is_fox_mode_read_only()



# -*- coding: utf-8 -*-
import pytest
from foxha.utils import Utils
import foxha.connection


@pytest.fixture
def logger():
    import logging
    return logging.getLogger(__name__)


def test_open_new_connection_succeds(test_key_path):
    assert foxha.connection.Connection(
        'localhost', 3306, '', 'root', '',
        Utils.parse_key_file(test_key_path)
    ).connect()


def test_connection_from_config_file():
    conn = foxha.connection.from_config_file()
    assert conn.connect()



import pytest
from cryptography.fernet import Fernet


def test_parse_key_file_successculy(utils, cipher_suite, test_key_path):
    assert isinstance(cipher_suite, Fernet)

    fernet_encrypt = Fernet(open(test_key_path, 'r').readline())
    assert cipher_suite._signing_key == fernet_encrypt._signing_key
    assert cipher_suite._encryption_key == fernet_encrypt._encryption_key


def test_parse_key_throws_IO_error(utils, capsys, connection_config_not_exist):
    file_path = connection_config_not_exist
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_key_file(file_path)

    error_message = u'\x1b[38;5;160mERROR: "{}" file does not exist or you'\
        ' don`t have permission to read it.\x1b[0m'.format(file_path)
    out, err = capsys.readouterr()

    assert error_message == out.rstrip()
    assert str(exec_info.value) == '3'


def test_parse_empty_key_throws_exception(utils, capsys,
    connection_config_empty
):
    file_path = connection_config_empty
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_key_file(keyfile=file_path)

    error_message = u'\x1b[38;5;160mERROR: {} may be empty because'\
        ' Fernet key must be 32 url-safe base64-encoded'\
        ' bytes.\x1b[0m\n'.format(file_path)

    out, err = capsys.readouterr()

    assert error_message == out
    assert str(exec_info.value) == '3'


def test_parse_empty_key_throws_type_error_exception(
    utils, capsys, connection_config_with_pading_token
):
    file_path = connection_config_with_pading_token
    with pytest.raises(SystemExit) as exec_info:
        utils.parse_key_file(keyfile=file_path)

    key = utils.get_key_value_from_key_file(file_path)
    error_message = u'\x1b[38;5;160mERROR: {} is not a valid key'\
        ' because Incorrect padding\x1b[0m\n'.format(key)

    out, err = capsys.readouterr()

    assert error_message == out
    assert str(exec_info.value) == '3'


def test_key_value_from_key_file_succeds(utils, test_key_path):
    key = utils.get_key_value_from_key_file(test_key_path)
    assert key == 'H-fx88gflMH13dsnbF9oH8EG261j9lxeWAFirHVMTi4='


def test_key_value_from_key_file_throws_exception(utils,
    connection_config_not_exist
):
    with pytest.raises(IOError):
        utils.get_key_value_from_key_file(connection_config_not_exist)




class ManyWriteNodesError(EnvironmentError):
    def __init__(self, group, nodes):
        msg = 'The group "{}" has {} write nodes. Should be just one'\
            .format(group, nodes)
        super(EnvironmentError, self).__init__(msg)


class NoWriteNodeError(EnvironmentError):
    def __init__(self, group):
        msg = 'The group "{}" don\'t have write node'.format(group)
        super(EnvironmentError, self).__init__(msg)


class NoReadNodeError(EnvironmentError):
    def __init__(self, group):
        msg = 'The group "{}" don\'t have read node'.format(group)
        super(EnvironmentError, self).__init__(msg)


class IsNotMasterMasterEnvironmentError(EnvironmentError):
    def __init__(self, group):
        msg = 'The group "{}" don\'t have read node'.format(group)
        super(EnvironmentError, self).__init__(msg)


class NodeIsDownError(EnvironmentError):
    def __init__(self, node):
        msg = 'Could not connect with node "{}"'.format(node)
        super(EnvironmentError, self).__init__(msg)


class ReplicationNotRunningError(EnvironmentError):
    def __init__(self, node):
        msg = 'The replication is stopped at node "{}"'.format(node)
        super(EnvironmentError, self).__init__(msg)


class NodeWithDelayError(EnvironmentError):
    def __init__(self, node, delay):
        msg = 'The node "{}" is {} seconds delayed'.format(node, delay)
        super(EnvironmentError, self).__init__(msg)


class GroupNotFoundError(EnvironmentError):
    def __init__(self, group):
        msg = 'Group "{}" not found'.format(group)
        super(EnvironmentError, self).__init__(msg)


class NodeNotFoundError(EnvironmentError):
    def __init__(self, node, group):
        msg = 'Node "{}" not found in group "{}"'.format(node, group)
        super(EnvironmentError, self).__init__(msg)


class GroupAlreadyAddedError(EnvironmentError):
    def __init__(self, group):
        msg = 'Group "{}" already added'.format(group)
        super(EnvironmentError, self).__init__(msg)


class NodeAlreadyAddedError(EnvironmentError):
    def __init__(self, node, group):
        msg = 'Node "{}" already added in group "{}"'.format(node, group)
        super(EnvironmentError, self).__init__(msg)


class GroupWithNodesError(EnvironmentError):
    def __init__(self, group, nodes):
        msg = 'The group "{}" has "{}" nodes'.format(group, nodes)
        super(EnvironmentError, self).__init__(msg)



import time
import pytest
from foxha.node import Node, PhysicalNode
from foxha.utils import Utils
import conftest
import test_utils


@pytest.fixture
def physical_master_node(test_key_path, environment):
    node = Node(
        environment.group, environment.master, conftest.default_connection()
    )

    return PhysicalNode(
        node.ip, node.port, node.mysql_user, node.mysql_password,
        Utils.parse_key_file(test_key_path)
    )


@pytest.fixture
def physical_replication_node(test_key_path, environment):
    node = Node(
        environment.group, environment.replication,
        conftest.default_connection()
    )

    return PhysicalNode(
        node.ip, node.port, node.mysql_user, node.mysql_password,
        Utils.parse_key_file(test_key_path)
    )


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.activate_nodes()
    test_utils.reset_database()


def setup_function(module):
    test_utils.activate_nodes()
    test_utils.reset_database()


def test_heartbeat(environment, capsys, physical_replication_node):
    test_utils.wait_for_replication_ok(environment, capsys)
    assert physical_replication_node.seconds_behind == 0


def test_heartbeat_with_delay(environment, physical_replication_node):
    test_utils.deactivate_node(environment.master_node)
    time.sleep(5)
    assert physical_replication_node.seconds_behind > 0


def test_mysql_status_up(environment, physical_master_node):
    assert physical_master_node.mysql_status == 'Up'
    assert physical_master_node.is_mysql_status_up()
    assert not physical_master_node.is_mysql_status_down()


def test_mysql_status_down(environment, physical_master_node):
    test_utils.deactivate_node(environment.master_node)
    assert physical_master_node.mysql_status == 'Down'
    assert not physical_master_node.is_mysql_status_up()
    assert physical_master_node.is_mysql_status_down()


def test_node_master_with_master_down(environment, physical_replication_node):
    test_utils.deactivate_node(environment.master_node)
    time.sleep(5)
    assert physical_replication_node.master


def test_node_mode(physical_master_node, physical_replication_node):
    assert physical_master_node.mode == 'read_write'
    assert physical_master_node.is_read_write()
    assert not physical_master_node.is_read_only()

    assert physical_replication_node.mode == 'read_only'
    assert physical_replication_node.is_read_only()
    assert not physical_replication_node.is_read_write()


def test_replication_status(physical_replication_node):
    assert physical_replication_node.replication_status == 'running'
    assert physical_replication_node.is_replication_running()
    assert not physical_replication_node.is_replication_stopped()


def test_replication_status_with_node_down(environment,
                                           physical_replication_node):
    test_utils.deactivate_node(environment.master_node)
    time.sleep(5)
    assert physical_replication_node.replication_status == 'stopped'
    assert physical_replication_node.is_replication_stopped()
    assert not physical_replication_node.is_replication_running()


def test_server_none_with_node_down(environment, physical_replication_node):
    test_utils.deactivate_node(environment.replication_node)

    assert physical_replication_node.mode == '-'
    assert physical_replication_node.replication_status == '-'
    assert physical_replication_node.seconds_behind == '-'
    assert not physical_replication_node.master
    assert not physical_replication_node.server_id
    assert not physical_replication_node.master_server_id



import argparse
from os import getenv
from .utils import Utils
from .print_format import print_warning


def arguments():

    parser = argparse.ArgumentParser(
        add_help=True,
        description="Description: MySQL FoxHA Crypt Passwords")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-g",
        "--generate_new_keyfile",
        help="generate new keyfile",
        action="store_true")
    group.add_argument(
        "-c",
        "--crypt",
        help="crypt password",
        action="store_true")
    group.add_argument(
        "-d",
        "--decrypt",
        help="decrypt password",
        action="store_true")
    parser.add_argument(
        "-p",
        "--password",
        help="specify password",
        action="store")
    parser.add_argument(
        "--keyfile",
        help="different path to key file - Default: ./config/.key",
        action="store")
    args = parser.parse_args()
    argument_vars = vars(args)

    # If None option is specified display help:
    argsvars = vars(parser.parse_args())
    if not any(argsvars.values()):
        parser.print_help()
        exit(0)

    # Parsing Env. Variable FOXHA_HOME
    FOXHA_HOME = getenv('FOXHA_HOME')

    if args.generate_new_keyfile:
        if not args.password:
            Utils.generate_new_keyfile(args.keyfile)
        else:
            print_warning("You should specify option [-g/--generate-new-keyfile] alone.")
        exit(1)

    # Key file argument
    if args.keyfile:
        cipher_suite = Utils.parse_key_file(args.keyfile)
    elif FOXHA_HOME:
        keyfile = FOXHA_HOME + '/config/.key'
        cipher_suite = Utils.parse_key_file(keyfile)
    else:
        cipher_suite = Utils.parse_key_file()

    if (args.crypt or args.decrypt) and args.password is None:
        print_warning("Please specify [-p/--password] parameter in conjunction with "\
            "[-c/--crypt] or [-d/--decrypt] options.")
        exit(1)

    if args.crypt:
        Utils.crypt_pass(cipher_suite, args.password)
        exit(1)
    if args.decrypt:
        Utils.decrypt_pass(cipher_suite, args.password)
        exit(1)

def main(args=None):
    arguments()

if __name__ == '__main__':
    main()



import os
import pytest
import ConfigParser
from foxha.utils import Utils


@pytest.fixture(scope='module')
def utils():
    return Utils()


@pytest.fixture(scope='module')
def cipher_suite(utils, test_key_path):
    return utils.parse_key_file(keyfile=test_key_path)


@pytest.fixture(scope='module')
def config_files_dir_utils(test_dir):
    return os.path.dirname(__file__) + '/config_files/'


@pytest.fixture(scope='module')
def connection_config_not_exist(config_files_dir_utils):
    return config_files_dir_utils + '.file_does_not_exists'


@pytest.fixture(scope='module')
def connection_config_empty(config_files_dir_utils):
    return config_files_dir_utils + '.test_empty_key'


@pytest.fixture(scope='module')
def connection_config_without_section(utils, config_files_dir_utils):
    return config_files_dir_utils + '.config_file_with_no_section_error.ini'


@pytest.fixture(scope='module')
def connection_config_without_option(utils, config_files_dir_utils):
    return config_files_dir_utils + '.config_file_with_no_option_error.ini'


@pytest.fixture(scope='module')
def connection_config_with_invalid_token(utils, config_files_dir_utils):
    return config_files_dir_utils + '.config_file_with_invalid_token.ini'


@pytest.fixture(scope='module')
def connection_config_with_pading_token(utils, config_files_dir_utils):
    return config_files_dir_utils + '.test_key_with_pading_error'

@pytest.fixture(scope='module')
def config_file_dict(test_connection_config_path):
    config = ConfigParser.ConfigParser()
    config.read(test_connection_config_path)

    return {
        'host': config.get('repository', 'Host'),
        'port': int(config.get('repository', 'Port')),
        'database': config.get('repository', 'Database'),
        'user': config.get('repository', 'User'),
        'password': config.get('repository', 'Pass')
    }



import node_utils
from query import Query
from errors import NoWriteNodeError, NoReadNodeError


class Group(object):

    def __init__(self, name, connection):
        self.name = name
        self.connection = connection

    def __config_from_fox_repository(self):
        return self.connection.execute(Query.SQL_CONFIG_GROUP % self.name)[0]

    @property
    def description(self):
        return self.__config_from_fox_repository()['description']

    @property
    def vip_address(self):
        return self.__config_from_fox_repository()['vip_address']

    @property
    def mysql_adm_user(self):
        return self.__config_from_fox_repository()['mysql_adm_user']

    @property
    def mysql_repl_user(self):
        return self.__config_from_fox_repository()['mysql_repl_user']

    @property
    def nodes(self):
        return node_utils.get_all_nodes(self.name, self.connection)

    @property
    def master_node(self):
        for node in self.nodes:
            if node.is_read_write():
                return node

        raise NoWriteNodeError(self.name)

    @property
    def replication_node(self):
        for node in self.nodes:
            if node.is_read_only():
                return node
        raise NoReadNodeError(self.name)

    def __eq__(self, other):
        return self.name == other.name



# -*- coding: utf-8 -*-

from os import sys
from os import getenv
import argparse
from foxha import __version__
from .print_format import print_warning
from .query import Query
from .utils import Utils
import formatter
import connection

# Initializing global constants
CIPHER_SUITE = None
LOGGER = None
CONNECTION = None


def check_node_exist(group_name, nodeip, plus_failed=False):
    return formatter.check_node_exist(
        group_name, nodeip, plus_failed, CONNECTION, LOGGER
    )


def set_read_only(group_name, nodeip):
    return formatter.set_read_only(group_name, nodeip, CONNECTION, LOGGER)


def set_read_write(group_name, nodeip):
    return formatter.set_read_write(group_name, nodeip, CONNECTION, LOGGER)


def switchover(group_name):
    formatter.switchover(group_name, CONNECTION, LOGGER)


def failover(group_name):
    formatter.failover(group_name, CONNECTION, LOGGER)


def set_status(group_name, nodeip, status):
    node_write = formatter.check_write(group_name, CONNECTION, LOGGER)
    if status == 'enabled':
        CONNECTION.query(Query.UPDATE_STATE % (status, nodeip, group_name))
        LOGGER.info(
            "Node: \"%s\" enabled at group_name: \"%s\"." %
            (nodeip, group_name))
    if node_write:
        if status == 'disabled':
            if node_write.ip == nodeip:
                print_warning(("The \"{}\" is the current read_write "\
                "node at group_name \"{}\" and cannot be disabled.").format(nodeip,\
                group_name))
                LOGGER.warning(
                    "The \"%s\" is the current read_write node at group_name \
                    \"%s\" and cannot be disabled." %
                    (nodeip, group_name))
            else:
                CONNECTION.query(Query.UPDATE_STATE % (status, nodeip, group_name))
                LOGGER.info(
                    "Node: \"%s\" disabled at group_name: \"%s\"." %
                    (nodeip, group_name))
        if status == 'failed':
            if node_write.ip == nodeip:
                failover(group_name)
            else:
                CONNECTION.query(Query.UPDATE_STATE % (status, nodeip, group_name))
    else:
        CONNECTION.query(Query.UPDATE_STATE % (status, nodeip, group_name))


def set(set, group_name, nodeip):
    if set == 'read_only':
        set_read_only(group_name, nodeip)
    elif set == 'read_write':
        set_read_write(group_name, nodeip)
    elif set == 'disabled' or set == 'failed' or set == 'enabled':
        set_status(group_name, nodeip, set)


def fox_arg_parse():
    parser = argparse.ArgumentParser(
        add_help=True,
        description="Description: MySQL FoxHA Administration")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '--version',
        action='version',
        version='%(prog)s ' +
        __version__)
    group.add_argument(
        "--status",
        help="show the status of a group_name",
        action="store_true")
    group.add_argument(
        "-l",
        "--list",
        help="lists the available group_names and nodes if [-g/--group] is specified",
        action="store_true")
    group.add_argument(
        "-c",
        "--config",
        help="show the config of a group_name",
        action="store_true")
    group.add_argument(
        "--start",
        help="check who is the read_write node and enable it at that node",
        action="store_true")
    group.add_argument(
        "--switchover",
        help="switchover to a new master",
        action="store_true")
    group.add_argument(
        "--failover",
        help="failover to a new master",
        action="store_true")
    group.add_argument(
        "--set",
        choices=[
            "read_write",
            "read_only",
            "failed",
            "disabled",
            "enabled"],
        action="store")
    parser.add_argument(
        "-g",
        "--group",
        metavar='GROUP_NAME',
        help="use to specify a group_name",
        action="store")
    parser.add_argument(
        "-n",
        "--nodeip",
        help="use to specify a node ip",
        action="store")
    parser.add_argument(
        "--keyfile",
        help="different path to key file - Default: ./config/.key",
        action="store")
    parser.add_argument(
        "--configfile",
        help="different path to config file - Default: ./config/foxha_config.ini",
        action="store")
    parser.add_argument(
        "--logfile",
        help="different path to log file - Default: ./log/foxha_[group_name].log",
        action="store")
    parser.add_argument(
        "--logretention",
        type=int,
        help="log file retention in days - Default: 4 days plus current",
        action="store")
    return parser


def main(values=None):
    args = fox_arg_parse().parse_args(values)
    argument_vars = vars(args)

    # Parsing Env. Variable FOXHA_HOME
    foxha_home = getenv('FOXHA_HOME')

    # Key file argument
    global CIPHER_SUITE
    if args.keyfile:
        CIPHER_SUITE = Utils.parse_key_file(args.keyfile)
    elif foxha_home:
        keyfile = foxha_home + '/config/.key'
        CIPHER_SUITE = Utils.parse_key_file(keyfile)
    else:
        CIPHER_SUITE = Utils.parse_key_file()

    # Config file argument
    config_file = './config/foxha_config.ini'
    if args.configfile:
        config_file = args.configfile
    elif foxha_home:
        config_file = foxha_home + '/config/foxha_config.ini'

    global CONNECTION
    CONNECTION = connection.from_config_file(CIPHER_SUITE, config_file)

    # Defining logfile name
    global LOGGER
    if args.group:
        logname = './log/foxha_' + args.group.lower() + '.log'
        if foxha_home:
            logname = foxha_home + '/log/foxha_' + \
                args.group.lower() + '.log'
    else:
        logname = '/tmp/foxha.log'

    if args.logfile and args.logretention:
        LOGGER = Utils.logfile(args.logfile, args.logretention)
    if args.logfile or args.logretention:
        if args.logfile:
            LOGGER = Utils.logfile(args.logfile)
        if args.logretention:
            LOGGER = Utils.logfile(logname, args.logretention)
    else:
        LOGGER = Utils.logfile(logname)

    if args.nodeip and args.set is None:
        print_warning("[-n/--node] specified out of context.")
        exit(1)

    if args.list:
        if args.group:
            formatter.list_nodes(args.group.lower(), CONNECTION)
        else:
            formatter.list_group(CONNECTION)

    if args.group:
        if argument_vars.values().count(True) == 0 and args.set is None:
            print_warning("You could not specify [-g/--group] alone")
            exit(1)
        else:
            arg_group_name = args.group.lower()
            if formatter.check_group_exist(arg_group_name, CONNECTION, LOGGER):
                pass
    else:
        print_warning("Group_name not specified. Use [-g/--group] to "\
        "specify the group_name")
        exit(2)

    if args.set:
        if args.nodeip:
            if args.set in ('disabled', 'enabled', 'failed', 'read_only'):
                if check_node_exist(arg_group_name, args.nodeip, plus_failed=True):
                    set(args.set, arg_group_name, args.nodeip)
            else:  # args.set in ('read_write')
                if check_node_exist(arg_group_name, args.nodeip, plus_failed=False):
                    set(args.set, arg_group_name, args.nodeip)
        else:
            print_warning("You must specify a [-n/--node] to the [--set] command.")
            exit(1)

    if args.status:
        formatter.status_nodes(arg_group_name, LOGGER, CONNECTION)

    if args.switchover:
        switchover(arg_group_name)

    if args.failover:
        failover(arg_group_name)

    if args.start:
        formatter.start(arg_group_name, CONNECTION, LOGGER)

    if args.config:
        formatter.config(arg_group_name, CONNECTION, LOGGER)


if __name__ == '__main__':
    main()



import time
import pytest
from foxha import fox
from foxha import inner_logic
from foxha.errors import NodeIsDownError
import test_utils


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.activate_nodes()
    test_utils.reset_database()
    time.sleep(30)


def setup_function(module):
    test_utils.activate_nodes()
    test_utils.reset_database()


def shut_down_master_node(environment):
    code, out, err = test_utils.deactivate_node(environment.master_node)
    assert code == 0


def start_up_master_node(environment):
    code, out, err = test_utils.activate_node(environment.master_node)
    assert code == 0


def test_cannot_change_to_write_when_replication_is_off(environment, capsys):
    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_only'

    shut_down_master_node(environment)
    node = test_utils.set_read_only(environment.replication, capsys)
    assert node['mode'] == 'read_only'


def test_cannot_change_to_read_when_node_is_down(environment, capsys):
    shut_down_master_node(environment)
    fox.set_read_only(environment.group, environment.master)

    message = '-set read_only failed. Connection could not be established ' \
              'with the node_ip %s' % environment.master
    out, err = capsys.readouterr()
    assert message in out


def test_can_failover_with_node_write_down(environment, capsys):
    shut_down_master_node(environment)
    fox.failover(environment.group)

    node = test_utils.get_node_from_status(environment.replication, capsys)
    assert node['mode'] == 'read_write'


def test_status_is_down_after_crash(environment, capsys):
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['status'] == 'Up'

    shut_down_master_node(environment)
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['status'] == 'Down'


def test_can_change_down_note_to_failed(environment, capsys):
    shut_down_master_node(environment)

    node = test_utils.failed_node(environment.master, capsys)
    assert node['status'] == 'failed'


def test_status_after_restart(environment, capsys):
    shut_down_master_node(environment)
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['status'] == 'Down'

    start_up_master_node(environment)
    node = test_utils.get_node_from_status(environment.master, capsys)
    assert node['status'] == 'Up'


def test_can_change_status_to_enabled_after_restart(environment, capsys):
    shut_down_master_node(environment)
    node = test_utils.failed_node(environment.master, capsys)
    assert node['status'] == 'failed'
    start_up_master_node(environment)

    node = test_utils.enable_node(environment.master, capsys)
    assert node['status'] == 'enabled'


def test_cannot_set_write_to_down_node(environment, default_connection):
    node = inner_logic.get_node(
        environment.group, environment.master, default_connection
    )
    shut_down_master_node(environment)

    with pytest.raises(NodeIsDownError):
        inner_logic.set_read_write(node, default_connection)


def test_no_master_master_with_down_node(environment, default_connection):
    shut_down_master_node(environment)

    assert not inner_logic.is_master_master(
        environment.group, default_connection
    )



from setuptools import setup, find_packages
import foxha

REQUIREMENTS_FILE = 'requirements.txt'


def parse_requirements():
    return [req[:-1] for req in open(REQUIREMENTS_FILE, 'r').readlines()]


setup(
    name="foxha",
    version=foxha.__version__,
    packages=find_packages(),
    author="Rafael Dantas Silva",
    author_email="rafael.dantas@corp.globo.com",
    description="A MySQL High Availability tool to replace flipper",
    license="GPL",
    keywords="tool mysql",
    url="https://github.com/globocom/foxha/",
    long_description="A MySQL High Availability tool to replace flipper",
    install_requires=parse_requirements(),

    extras_require={
        'dev': [],
        'test': [],
    },

    entry_points={
        'console_scripts': [
            'fox = foxha.fox:main',
            'fox_crypt = foxha.crypt:main',
        ],
    },

)



import pytest
from foxha.group import Group
import foxha.fox
import test_utils
from foxha.errors import NoWriteNodeError, NoReadNodeError


def setup_module(module):
    test_utils.connect_database()


def teardown_module(module):
    test_utils.reset_database()


def setup_function(module):
    test_utils.reset_database()


@pytest.fixture
def main_group(environment, default_connection):
    return Group(environment.group, default_connection)


def test_can_create_group(environment, main_group):
    assert main_group.name == environment.group
    assert main_group.description == environment.description
    assert main_group.vip_address == environment.vip_address
    assert main_group.mysql_adm_user == environment.mysql_adm_user
    assert main_group.mysql_repl_user == environment.mysql_repl_user
    assert len(main_group.nodes) == len(environment.nodes)
    assert main_group.master_node.ip == environment.master
    assert main_group.replication_node.ip == environment.replication


def test_no_master_node(environment, main_group):
    foxha.fox.set_read_only(environment.group, environment.master)
    with pytest.raises(NoWriteNodeError):
        assert environment.master == main_group.master_node.ip


def test_no_replication_node(environment, main_group):
    foxha.fox.set('failed', environment.group, environment.master)
    foxha.fox.set_read_write(environment.group, environment.replication)
    with pytest.raises(NoReadNodeError):
        assert environment.replication == main_group.replication_node.ip



from query import Query
from node import Node


def get_all_nodes(group, connection):
    result = connection.execute(Query.SQL_LIST_NODES % group)
    return [Node(group, node['node_ip'], connection)
            for node in result]



from datetime import datetime
from _mysql import OperationalError
from query import Query
from connection import Connection


READ_ONLY_MYSQL_STATUS = 'ON'
READ_WRITE_STATUS = 'read_write'
READ_ONLY_STATUS = 'read_only'
MYSQL_SERVER_STATUS_UP = 'Up'
MYSQL_SERVER_STATUS_DOWN = 'Down'
REPLICATION_STATUS_RUNNING = 'running'
REPLICATION_STATUS_STOPPED = 'stopped'
SLAVE_RUNNING_STATUS = 'Yes'


class LogicalNode(object):
    def __init__(self, group, ip, name, fox_connection):
        self.group = group
        self.ip = ip
        self.name = name
        self.address = self.format_server_name(ip, name)
        self.fox_connection = fox_connection

    def format_server_name(self, ip, name):
        return '{}({})'.format(name, ip)

    @property
    def fox_status(self):
        result = self.fox_connection.execute(
            Query.SQL_NODE_STATUS % (self.group, self.ip)
        )
        return result[0]['status']

    def is_enabled(self):
        return self.fox_status == 'enabled'

    def is_disabled(self):
        return self.fox_status == 'disabled'

    def is_failed(self):
        return self.fox_status == 'failed'

    @property
    def fox_mode(self):
        return self.fox_connection.execute(
            Query.SQL_NODE_STATUS % (self.group, self.ip)
        )[0]['mode']

    def is_fox_mode_read_only(self):
        return self.fox_mode == READ_ONLY_STATUS

    def is_fox_mode_read_write(self):
        return self.fox_mode == READ_WRITE_STATUS

    def __eq__(self, other):
        return (self.group == other.group and self.ip == other.ip)


class PhysicalNode(object):
    def __init__(self, ip, port, user, password, cipher):
        self.node_connection = Connection(ip, port, '', user, password, cipher)
        self.mysql_user = user
        self.mysql_password = password

    @property
    def mysql_status(self):
        try:
            with self.node_connection.connect():
                return MYSQL_SERVER_STATUS_UP
        except:
            return MYSQL_SERVER_STATUS_DOWN

    def is_mysql_status_up(self):
        return self.mysql_status == MYSQL_SERVER_STATUS_UP

    def is_mysql_status_down(self):
        return self.mysql_status == MYSQL_SERVER_STATUS_DOWN

    @property
    def mode(self):
        try:
            result = self.node_connection.execute(Query.SHOW_GLOBAL_STATUS_READ)[0]
        except OperationalError:
            return '-'
        else:
            if result['Value'] == READ_ONLY_MYSQL_STATUS:
                return READ_ONLY_STATUS
            return READ_WRITE_STATUS

    def is_read_only(self):
        return self.mode == READ_ONLY_STATUS

    def is_read_write(self):
        return self.mode == READ_WRITE_STATUS

    @property
    def replication_status(self):
        try:
            result = self.node_connection.execute(Query.SHOW_SLAVE_STATUS)[0]
        except OperationalError:
            return '-'
        else:
            is_io_running = result['Slave_IO_Running'] == SLAVE_RUNNING_STATUS
            is_sql_running = result['Slave_SQL_Running'] == SLAVE_RUNNING_STATUS
            if is_io_running and is_sql_running:
                return REPLICATION_STATUS_RUNNING
            return REPLICATION_STATUS_STOPPED

    def is_replication_stopped(self):
        return self.replication_status == REPLICATION_STATUS_STOPPED

    def is_replication_running(self):
        return self.replication_status == REPLICATION_STATUS_RUNNING

    @property
    def seconds_behind(self):
        try:
            result = self.node_connection.execute(Query.SQL_STATUS_HEARTBEAT)[0]
        except OperationalError:
            return '-'
        else:
            date_format = '%Y-%m-%d %H:%M:%S'
            now = datetime.now().replace(microsecond=0)

            heartbeat_time = datetime.strptime(
                result['last_update'], date_format
            )
            return (now - heartbeat_time).total_seconds()

    @property
    def master(self):
        try:
            result = self.node_connection.execute(Query.SHOW_SLAVE_STATUS)[0]
        except OperationalError:
            return None
        else:
            return result['Master_Host'] + ':' + str(result['Master_Port'])

    @property
    def server_id(self):
        try:
            result = self.node_connection.execute(Query.SHOW_GLOBAL_SERVER_ID)[0]
        except OperationalError:
            return None
        else:
            return int(result['Value'])

    @property
    def master_server_id(self):
        try:
            result = self.node_connection.execute(Query.SHOW_SLAVE_STATUS)[0]
        except OperationalError:
            return None
        else:
            return int(result['Master_Server_Id'])


class Node(LogicalNode, PhysicalNode):
    def __init__(self, group, ip, fox_connection):

        result = fox_connection.execute(Query.SQL_NODE_DETAILS % (group, ip))[0]

        LogicalNode.__init__(
            self, group, ip, result['servername'], fox_connection
        )
        PhysicalNode.__init__(
            self, ip, result['node_port'], result['mysql_adm_user'],
            fox_connection.cipher.decrypt(result['mysql_adm_pass']),
            fox_connection.cipher
        )

        self.port = result['node_port']

    def get_name_ip_port(self):
        return '{}({}:{})'.format(self.name, self.ip, self.port)
